## Summary

---

**TODO** This originally was in `gltfTutorial_006_MaterialsTechniquesProgramsShaders.md`

---

### Rendering an object with a given material


This section will illustrate the process of rendering an object that is part of a glTF asset. At some points, this description will refer to other sections, where each required concept is explained in more detail. Also, the description here is meant to be *illustrative*. There are several degrees of freedom for an actual implementation, and to keep the code as simple as possible, some functions are written in pseudocode and most error checks are omitted. The main goal of this section is to show how the elements that are found in a glTF JSON file have to be combined to render the objects with OpenGL.

**TODO** Describe the process once - as text, bulletpoints, pseudocode or image?

* To render an object with material `exampleMaterial`, first find the technique of the material
* Enable the GL program that corresponds to the program of the technique 
* For each attribute of the technique: Enable attribute location
* For each uniform of the technique: 
 * Somewhere get the uniform location  
 * Fetch uniform value
 * Set uniform value  
* Render. (<- don't forget this ;-))

---
---

---

**TODO** This originally was in `gltfTutorial_007_BuffersBufferViewsAccessors.md`

---


### Using buffer views and accessors in OpenGL
   
The glTF JSON basically contains a description of the parameters for the relevant OpenGL function calls. To illustrate this, here are some basic examples showing the relevant function calls. 

#### Uploading `bufferView` data into an OpenGL buffer

When the data of a `bufferView` should be passed to OpenGL, then this can be done with the [`bufferData`](https://developer.mozilla.org/en-US/docs/Web/API/WebGLRenderingContext/bufferData) function. This function expects a `target` as the first parameter, which can either be `GL_ARRAY_BUFFER` or `GL_ELEMENT_ARRAY_BUFFER`, depending on whether the data is used for vertex attributes or for indices. The `target` is stored in the `bufferView` as well, and can simply be passed to the `bufferData` function:

    // The bufferView whose data should be passed to OpenGL
    var bufferView = ...; 
 
    // The bufferView refers to a buffer. The data of this buffer is
    // read from a ".bin" file into an ArrayBuffer 
    var bufferData = fetchDataFor(bufferView.buffer); 

    // Create a DataView that describes the part of the buffer
    // that is defined by the bufferView. The byteOffset and
    // the byteLength define a "slice" of the whole data:
    var dataView = new DataView(bufferData, bufferView.byteOffset, bufferView.byteLength);

    // Create an OpenGL buffer ID, and bind the GL buffer
    var glBuffer = gl.createBuffer();
    gl.bindBuffer(bufferView.target, glBuffer);

    // Fill the GL buffer with the data from the DataView
    gl.bufferData(bufferView.target, dataView, gl.STATIC_DRAW);

After this data has been uploaded into the OpenGL buffer, it may be used in rendering calls.

**TODO** Examples in LWJGL/JOGL, GL, ... ?
 

#### Using an `accessor` to define buffer data as an OpenGL attribute 

The following lines of code illustrate how the data of an OpenGL buffer may be used as a vertex attribute in a rendering call. As described in    

    var techniqueParametersId = technique.attributes[attributeName];
    var techniqueParameters = technique.parameters[techniqueParametersId];
    var semantic = techniqueParameters.semantic;

    var accessorId = meshPrimitive.attributes[semantic];
    var accessor = gltf.accessors[accessorId];
    var bufferViewId = accessor.bufferView;
    var glBuffer = fetchGlBufferFor(bufferViewId);

    var attributeLocation = gl.getAttribLocation(glProgram, attributeName);
    var numComponents = getNumComponentsForAccessorType(accessor.type);

    gl.bindBuffer(gl.ARRAY_BUFFER, glBuffer);
    gl.enableVertexAttribArray(attributeLocation);
    gl.vertexAttribPointer(attributeLocation, numComponents, 
        accessor.componentType, false, accessor.byteStride, accessor.byteOffset);


**TODO** Examples in LWJGL/JOGL, GL, ... ?


---
---


---

**TODO** This originally was in `gltfTutorial_005_MeshesTexturesImagesSamplers.md`

---

After the image data has been read, it can be passed to OpenGL using the `glTexImage2D` function. The remaining properties of a `texture` are simply the other parameters that have to be passed to the `glTexImage2D` function call. Similarly, the `sampler` contains the properties that are passed to `glTexParameter` function calls, in order to define the sampling- and clamping behavior of the texture. 

So assuming that the actual data for an image has been read, and may be looked up using the ID of the image, the pseudocode for creating the OpenGL texture from the glTF `texture` is as follows:

    var texture = gltf.textures[i];

    // Create and bind the OpenGL texture
    var glTexture = gl.createTexture();
    gl.bindTexture(texture.target, glTexture);

    // Upload the image data for the glTF texture image to OpenGL
    var imageData = lookUpImageDataFor(texture.image);
    gl.texImage2D(texture.target, 0, texture.internalFormat, texture.format, texture.type, imageData);

    // Set the OpenGL texture parameters based on the glTF texture sampler
    gl.texParameteri(texture.target, gl.TEXTURE_MAG_FILTER, texture.sampler.magFilter);
    gl.texParameteri(texture.target, gl.TEXTURE_MIN_FILTER, texture.sampler.minFilter);
    gl.texParameteri(texture.target, gl.TEXTURE_WRAP_S, texture.sampler.wrapS);
    gl.texParameteri(texture.target, gl.TEXTURE_WRAP_T, texture.sampler.wrapT);
    


> Implementation notes:
> 
> * **TODO** Examples in WebGL, LWJGL, JOGL, GL, ...
> * **TODO** Mention gl.generateMipmap(target);
