var gltfTutorial = gltfTutorial || {};

// Note: All this does not work on IE. Finding a solution for that is no fun. 
// (They REALLY and TOTALLY messed this up, didn't they?)

(function(gltfTutorial) {
    'use strict';

    var IoUtils = {

        /**
         * Resolve the given relative URI against the base URI
         *
         * @param {string} baseUri The URI to resolve against
         * @param {string} relativeUri The relative URI
         * @return {string} The resolved URI
         */
        resolveUri : function(baseUri, relativeUri) {
            // Adapted from http://stackoverflow.com/a/14780463
            var stack = baseUri.split("/");
            var parts = relativeUri.split("/");
            for (var i = 0; i < parts.length; i++) {
                if (parts[i] == "..")
                    stack.pop();
                else if (parts[i] != ".")
                    stack.push(parts[i]);
            }
            return stack.join("/");
        },

        /**
         * Loads a "application/json" file with the given name, and returns
         * a Promise for the JSON object that was read from the file
         *
         * @param {string} fileName The name of the file to load
         * @return {Promise} The promise for the JSON object
         */
        loadJsonFile : function(fileName) {
            return new Promise(function(resolve, reject) {
                var request = new XMLHttpRequest();
                request.open('GET', fileName, true);
                request.overrideMimeType("application/json");
                request.responseType = "json";
                request.onload = function() {
                    if (request.status == "200") {
                        var result = request.response;
                        if (request.responseType != "json") {
                            // IE cr4p
                            result = JSON.parse(request.responseText);
                        }
                        resolve(result);
                    } else {
                        reject(Error(request.statusText));
                    }
                };
                request.onerror = function() {
                    reject(Error("Network Error"));
                };
                request.send(null);
            });
        },

        /**
         * Loads a "text/plain" file with the given name, and returns
         * a Promise for the contents of the file as a string
         *
         * @param {string} fileName The name of the file to load
         * @return {Promise} The promise for the file contents
         */
        loadTextFile : function(fileName) {
            return new Promise(function(resolve, reject) {
                var request = new XMLHttpRequest();
                request.overrideMimeType("text/plain");
                request.open('GET', fileName, true);
                request.onload = function() {
                    if (request.status == "200") {
                        resolve(request.responseText);
                    } else {
                        reject(Error(request.statusText));
                    }
                };
                request.onerror = function() {
                    reject(Error("Network Error"));
                };
                request.send(null);
            });
        },

        /**
         * Loads a binary file with the given name, and returns
         * a Promise for the binary (arraybuffer) contents of
         * the file
         *
         * @param {string} fileName The name of the file to load
         * @return {Promise} The promise for the binary file contents
         */
        loadBinaryFile : function(fileName) {
            return new Promise(function(resolve, reject) {
                var request = new XMLHttpRequest();
                request.responseType = "arraybuffer";
                request.open('GET', fileName, true);
                request.onload = function() {
                    if (request.status == "200") {
                        resolve(request.response);
                    } else {
                        reject(Error(request.statusText));
                    }
                };
                request.onerror = function() {
                    reject(Error("Network Error"));
                };
                request.send(null);
            });
        }
    };
    
    gltfTutorial.IoUtils = IoUtils;

})(gltfTutorial);

