var gltfTutorial = gltfTutorial || {};

define([
    'Utils'
], 
function (
    Utils
){
    
    (function(gltfTutorial) {
    
        'use strict';
    
        var isUndefined = gltfTutorial.Utils.isUndefined;        
    
    
        /**
         * Returns a string representation of the given number, with
         * the given number of digits and at least the given total
         * length
         *
         * @param {Number} number The number
         * @param {Number} decimals The number of decimal places
         * @param {Number} length The minimum total length of the result string
         * @return {String} The string
         */
        var formatNumber = function(number, decimals, length) {
            var result = number.toFixed(decimals);
            while (result.length < length) {
                result = " " + result;
            }
            return result;
        };
    
        /**
         * Returns a tabular string representation of the given array,
         * formatting the array as a matrix with the given number of
         * columns, in column-major order
         *
         * @param {Object} array The array
         * @param {Number} columns The number of columns
         * @param {Number} decimals The number of decimal places
         * @param {Number} length The minimum total length of the result string
         * @return {String} The string
         */
        function arrayToStringColumnMajor2D(array, columns, decimals, length) {
            var rows = Math.ceil(array.length / columns);
            var result = "";
            for (var r = 0; r < rows; r++) {
                for (var c = 0; c < columns; c++) {
                    if (c > 0) {
                        result += " ";
                    }
                    var index = r + c * rows;
                    if (index < array.length)
                    {
                        result += formatNumber(array[index], decimals, length);
                    }
                }
                result += "\n";
            }
            return result;
        };
    
        var MatUtils = {
    
            /**
             * Prints a log message with the given matrix (which must be some
             * sort of array, with a length property and indexed access). 
             * Details are not specified, but this may only be implemented
             * properly for 3x3 and 4x4 matrices. 
             * 
             * @param {Object} matrix The matrix
             */
            logMatrix : function(matrix) {
                if (isUndefined(matrix.length)) {
                    console.log("Not a matrix: " + matrix);
                    return;
                }
                var decimals = 2;
                var length = 7;
                if (matrix.length === 9) {
                    console.log(arrayToStringColumnMajor2D(
                        matrix, 3, decimals, length));
                    return;
                }
                if (matrix.length === 16) {
                    console.log(arrayToStringColumnMajor2D(
                        matrix, 4, decimals, length));
                    return;
                }
                console.log("Invalid matrix: " + matrix);
            } 
        };
        
        gltfTutorial.MatUtils = MatUtils;
        
    })(gltfTutorial);
    
});